@@ -1,4 +1,4 @@
-/*      $Id: frame.h 24716 2007-01-24 21:42:47Z olivier $
+/*      $Id$
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,10 +12,12 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
+
 
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
+        xfwm4    - (c) 2002-2009 Olivier Fourdan
 
  */
 
@@ -23,20 +25,13 @@
 #define INC_FRAME_H
 
 #ifdef HAVE_CONFIG_H
-#  include "config.h"
+#include "config.h"
 #endif
 
 #include "screen.h"
 #include "mypixmap.h"
 #include "client.h"
 
-struct _FramePixmap
-{
-    xfwmPixmap pm_title;
-    xfwmPixmap pm_sides[3];
-};
-typedef struct _FramePixmap FramePixmap;
-
 int                      frameDecorationLeft                    (ScreenInfo *);
 int                      frameDecorationRight                   (ScreenInfo *);
 int                      frameDecorationTop                     (ScreenInfo *);
@@ -49,10 +44,10 @@ int                      frameX                                 (Client *);
 int                      frameY                                 (Client *);
 int                      frameWidth                             (Client *);
 int                      frameHeight                            (Client *);
-void                     frameDraw                              (Client *, 
-                                                                 gboolean);
 void                     frameClearQueueDraw                    (Client *);
-void                     frameQueueDraw                         (Client *);
-
+void                     frameQueueDraw                         (Client *,
+                                                                 gboolean);
+void                     frameDraw                              (Client *,
+                                                                 gboolean);
 
 #endif /* INC_FRAME_H */
