@@ -1,4 +1,4 @@
-/*      $Id: hints.c 24107 2006-12-14 21:49:51Z olivier $
+/*      $Id$
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,16 +12,18 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
+
 
         oroborus - (c) 2001 Ken Lynch
         Metacity - (c) 2001 Havoc Pennington
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
+        xfwm4    - (c) 2002-2009 Olivier Fourdan
 
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <X11/X.h>
@@ -73,24 +75,6 @@ internal_utf8_strndup (const gchar *src, gssize max_len)
     return g_strndup (src, s - src);
 }
 
-static gchar*
-create_name_with_host (DisplayInfo *display_info, const gchar *name, const gchar *hostname)
-{
-    gchar *title;
-
-    if (strlen (hostname) && (display_info->hostname) && (g_strcasecmp (display_info->hostname, hostname)))
-    {
-        /* TRANSLATORS: "(on %s)" is like "running on" the name of the other host */
-        title = g_strdup_printf (_("%s (on %s)"), name, hostname);
-    }
-    else
-    {
-        title = g_strdup (name);
-    }
-
-    return title;
-}
-
 unsigned long
 getWMState (DisplayInfo *display_info, Window w)
 {
@@ -194,6 +178,11 @@ getWMProtocols (DisplayInfo *display_info, Window w)
             {
                 result |= WM_PROTOCOLS_CONTEXT_HELP;
             }
+            /* Ping */
+            if (*ap == display_info->atoms[NET_WM_PING])
+            {
+                result |= WM_PROTOCOLS_PING;
+            }
         }
         if (protocols)
         {
@@ -397,7 +386,7 @@ setGnomeProtocols (DisplayInfo *display_info, Window root, Window w)
 void
 setNetSupportedHint (DisplayInfo *display_info, Window root, Window check_win)
 {
-    Atom atoms[64];
+    Atom atoms[ATOM_COUNT];
     unsigned long data[1];
     int i;
 
@@ -412,26 +401,36 @@ setNetSupportedHint (DisplayInfo *display_info, Window root, Window check_win)
     atoms[i++] = display_info->atoms[NET_DESKTOP_NAMES];
     atoms[i++] = display_info->atoms[NET_DESKTOP_VIEWPORT];
     atoms[i++] = display_info->atoms[NET_FRAME_EXTENTS];
+    atoms[i++] = display_info->atoms[NET_MOVERESIZE_WINDOW];
     atoms[i++] = display_info->atoms[NET_NUMBER_OF_DESKTOPS];
     atoms[i++] = display_info->atoms[NET_REQUEST_FRAME_EXTENTS];
     atoms[i++] = display_info->atoms[NET_SHOWING_DESKTOP];
     atoms[i++] = display_info->atoms[NET_SUPPORTED];
     atoms[i++] = display_info->atoms[NET_SUPPORTING_WM_CHECK];
+    atoms[i++] = display_info->atoms[NET_SYSTEM_TRAY_OPCODE];
+    atoms[i++] = display_info->atoms[NET_WM_ACTION_ABOVE];
+    atoms[i++] = display_info->atoms[NET_WM_ACTION_BELOW];
     atoms[i++] = display_info->atoms[NET_WM_ACTION_CHANGE_DESKTOP];
     atoms[i++] = display_info->atoms[NET_WM_ACTION_CLOSE];
+    atoms[i++] = display_info->atoms[NET_WM_ACTION_FULLSCREEN];
     atoms[i++] = display_info->atoms[NET_WM_ACTION_MAXIMIZE_HORZ];
     atoms[i++] = display_info->atoms[NET_WM_ACTION_MAXIMIZE_VERT];
+    atoms[i++] = display_info->atoms[NET_WM_ACTION_MINIMIZE];
     atoms[i++] = display_info->atoms[NET_WM_ACTION_MOVE];
     atoms[i++] = display_info->atoms[NET_WM_ACTION_RESIZE];
     atoms[i++] = display_info->atoms[NET_WM_ACTION_SHADE];
     atoms[i++] = display_info->atoms[NET_WM_ACTION_STICK];
     atoms[i++] = display_info->atoms[NET_WM_ALLOWED_ACTIONS];
+    atoms[i++] = display_info->atoms[NET_WM_CONTEXT_HELP];
     atoms[i++] = display_info->atoms[NET_WM_DESKTOP];
+    atoms[i++] = display_info->atoms[NET_WM_FULLSCREEN_MONITORS];
     atoms[i++] = display_info->atoms[NET_WM_ICON];
     atoms[i++] = display_info->atoms[NET_WM_ICON_GEOMETRY];
     atoms[i++] = display_info->atoms[NET_WM_ICON_NAME];
     atoms[i++] = display_info->atoms[NET_WM_MOVERESIZE];
     atoms[i++] = display_info->atoms[NET_WM_NAME];
+    atoms[i++] = display_info->atoms[NET_WM_PID];
+    atoms[i++] = display_info->atoms[NET_WM_PING];
     atoms[i++] = display_info->atoms[NET_WM_STATE];
     atoms[i++] = display_info->atoms[NET_WM_STATE_ABOVE];
     atoms[i++] = display_info->atoms[NET_WM_STATE_BELOW];
@@ -450,6 +449,9 @@ setNetSupportedHint (DisplayInfo *display_info, Window root, Window check_win)
     atoms[i++] = display_info->atoms[NET_WM_SYNC_REQUEST];
     atoms[i++] = display_info->atoms[NET_WM_SYNC_REQUEST_COUNTER];
     atoms[i++] = display_info->atoms[NET_WM_USER_TIME];
+    atoms[i++] = display_info->atoms[NET_WM_USER_TIME_WINDOW];
+    atoms[i++] = display_info->atoms[NET_WM_WINDOW_OPACITY];
+    atoms[i++] = display_info->atoms[NET_WM_WINDOW_OPACITY_LOCKED];
     atoms[i++] = display_info->atoms[NET_WM_WINDOW_TYPE];
     atoms[i++] = display_info->atoms[NET_WM_WINDOW_TYPE_DESKTOP];
     atoms[i++] = display_info->atoms[NET_WM_WINDOW_TYPE_DIALOG];
@@ -463,7 +465,10 @@ setNetSupportedHint (DisplayInfo *display_info, Window root, Window check_win)
 #ifdef HAVE_LIBSTARTUP_NOTIFICATION
     atoms[i++] = display_info->atoms[NET_STARTUP_ID];
 #endif
-    g_assert (i < 64);
+#ifdef ENABLE_KDE_SYSTRAY_PROXY
+    atoms[i++] = display_info->atoms[KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR];
+#endif
+    g_assert (i < ATOM_COUNT);
     data[0] = check_win;
     XChangeProperty (display_info->dpy, root, display_info->atoms[NET_SUPPORTED],
                      XA_ATOM, 32, PropModeReplace, (unsigned char *) atoms, i);
@@ -585,6 +590,20 @@ setNetFrameExtents (DisplayInfo *display_info, Window w, int top, int left, int
                      XA_CARDINAL, 32, PropModeReplace, (unsigned char *) data, 4);
 }
 
+void
+setNetFullscreenMonitors (DisplayInfo *display_info, Window w, gint top, gint bottom, gint left, gint right)
+{
+    unsigned long data[4] = { 0, 0, 0, 0 };
+
+    TRACE ("entering setNetFullscreenMonitors");
+    data[0] = (unsigned long) top;;
+    data[1] = (unsigned long) bottom;
+    data[2] = (unsigned long) left;
+    data[3] = (unsigned long) right;
+    XChangeProperty (display_info->dpy, w, display_info->atoms[NET_WM_FULLSCREEN_MONITORS],
+                     XA_CARDINAL, 32, PropModeReplace, (unsigned char *) data, 4);
+}
+
 void
 initNetDesktopInfo (DisplayInfo *display_info, Window root, int workspace, int width, int height)
 {
@@ -611,7 +630,7 @@ setUTF8StringHint (DisplayInfo *display_info, Window w, int atom_id, const gchar
 
     XChangeProperty (display_info->dpy, w, display_info->atoms[atom_id],
                      display_info->atoms[UTF8_STRING], 8, PropModeReplace,
-                     (unsigned char *) val, strlen (val) + 1);
+                     (unsigned char *) val, strlen (val));
 }
 
 void
@@ -703,6 +722,7 @@ getUTF8StringData (DisplayInfo *display_info, Window w, int atom_id, gchar **str
     unsigned long n_items;
 
     g_return_val_if_fail (((atom_id >= 0) && (atom_id < ATOM_COUNT)), FALSE);
+
     TRACE ("entering getUTF8StringData");
 
     *str_p = NULL;
@@ -736,6 +756,7 @@ getUTF8String (DisplayInfo *display_info, Window w, int atom_id, gchar **str_p,
     char *xstr;
 
     g_return_val_if_fail (((atom_id >= 0) && (atom_id < ATOM_COUNT)), FALSE);
+
     TRACE ("entering getUTF8String");
 
     if (!getUTF8StringData (display_info, w, atom_id, &xstr, length))
@@ -810,6 +831,7 @@ getUTF8StringList (DisplayInfo *display_info, Window w, int atom_id, gchar ***st
         if (g_utf8_validate (ptr, -1, NULL))
         {
             retval[i] = internal_utf8_strndup (ptr, MAX_STR_LENGTH);
+            xfce_utf8_remove_controls((gchar *) retval[i], -1, NULL);
         }
         else
         {
@@ -825,16 +847,51 @@ getUTF8StringList (DisplayInfo *display_info, Window w, int atom_id, gchar ***st
 }
 
 gboolean
-getClientMachine (DisplayInfo *display_info, Window w, gchar **machine)
+getWindowProp (DisplayInfo *display_info, Window window, int atom_id, Window *w)
+{
+    Atom type;
+    int format;
+    unsigned long nitems;
+    unsigned long bytes_after;
+    unsigned char *prop;
+
+    TRACE ("entering getWindowProp");
+
+    g_return_val_if_fail (window != None, FALSE);
+    g_return_val_if_fail (((atom_id >= 0) && (atom_id < ATOM_COUNT)), FALSE);
+
+    *w = None;
+    if (XGetWindowProperty (display_info->dpy, window, display_info->atoms[atom_id],
+                            0L, 1L, FALSE, XA_WINDOW, &type, &format, &nitems,
+                            &bytes_after, (unsigned char **) &prop) == Success)
+    {
+        if (prop)
+        {
+            *w = *((Window *) prop);
+            XFree (prop);
+        }
+        if (!prop || !check_type_and_format (32, XA_WINDOW, -1, format, type))
+        {
+            *w = None;
+            return FALSE;
+        }
+    }
+
+    return TRUE;
+}
+
+gboolean
+getWindowHostname (DisplayInfo *display_info, Window w, gchar **machine)
 {
     char *str;
     gboolean status;
 
-    TRACE ("entering getClientMachine");
+    TRACE ("entering getWindowHostname");
 
     g_return_val_if_fail (machine != NULL, FALSE);
     *machine = NULL;
     g_return_val_if_fail (w != None, FALSE);
+    g_return_val_if_fail (display_info != NULL, FALSE);
 
     status = FALSE;
     str = get_text_property (display_info, w, display_info->atoms[WM_CLIENT_MACHINE]);
@@ -852,36 +909,31 @@ getClientMachine (DisplayInfo *display_info, Window w, gchar **machine)
 }
 
 gboolean
-getWindowName (DisplayInfo *display_info, Window w, gchar **title)
+getWindowName (DisplayInfo *display_info, Window w, gchar **name)
 {
     char *str;
     int len;
-    gchar *machine;
-    gchar *name;
     gboolean status;
 
     TRACE ("entering getWindowName");
 
-    g_return_val_if_fail (title != NULL, FALSE);
-    *title = NULL;
+    g_return_val_if_fail (name != NULL, FALSE);
+    *name = NULL;
     g_return_val_if_fail (w != None, FALSE);
 
     status = FALSE;
-    getClientMachine (display_info, w, &machine);
     if (getUTF8StringData (display_info, w, NET_WM_NAME, &str, &len) ||
         (str = get_text_property (display_info, w, XA_WM_NAME)))
     {
-        name = internal_utf8_strndup (str, MAX_STR_LENGTH);
-        *title = create_name_with_host (display_info, name, machine);
-        g_free (name);
+        *name = internal_utf8_strndup (str, MAX_STR_LENGTH);
+        xfce_utf8_remove_controls(*name, -1, NULL);
         XFree (str);
         status = TRUE;
     }
     else
     {
-        *title = g_strdup ("");
+        *name = g_strdup ("");
     }
-    g_free (machine);
     return status;
 }
 
@@ -917,36 +969,17 @@ Window
 getClientLeader (DisplayInfo *display_info, Window window)
 {
     Window client_leader;
-    Atom actual_type;
-    int actual_format;
-    unsigned long nitems;
-    unsigned long bytes_after;
-    unsigned char *prop;
-
     TRACE ("entering getClientLeader");
 
     g_return_val_if_fail (window != None, None);
 
     client_leader = None;
-    if (XGetWindowProperty (display_info->dpy, window, display_info->atoms[WM_CLIENT_LEADER],
-                            0L, 1L, FALSE, AnyPropertyType, &actual_type, &actual_format, &nitems,
-                            &bytes_after, (unsigned char **) &prop) == Success)
-    {
-        if ((prop) && (actual_type == XA_WINDOW) && (actual_format == 32)
-            && (nitems == 1) && (bytes_after == 0))
-        {
-            client_leader = *((Window *) prop);
-        }
-        if (prop)
-        {
-            XFree (prop);
-        }
-    }
+    getWindowProp (display_info, window, WM_CLIENT_LEADER, &client_leader);
     return client_leader;
 }
 
 gboolean
-getNetWMUserTime (DisplayInfo *display_info, Window window, Time *time)
+getNetWMUserTime (DisplayInfo *display_info, Window window, guint32 *time)
 {
     Atom actual_type;
     int actual_format;
@@ -965,16 +998,32 @@ getNetWMUserTime (DisplayInfo *display_info, Window window, Time *time)
         if ((data) && (actual_type == XA_CARDINAL)
             && (nitems == 1) && (bytes_after == 0))
         {
-            *time = *((long *) data);
+            *time = *((guint32 *) data);
             XFree (data);
             return TRUE;
         }
     }
-    *time = (Time) 0;
+    *time = 0;
 
     return FALSE;
 }
 
+Window
+getNetWMUserTimeWindow (DisplayInfo *display_info, Window window)
+{
+    Window user_time_win;
+    TRACE ("entering getNetWMUserTimeWindow");
+
+    g_return_val_if_fail (window != None, None);
+
+    user_time_win = None;
+    if (getWindowProp (display_info, window, NET_WM_USER_TIME_WINDOW, &user_time_win))
+    {
+        return user_time_win;
+    }
+    return window;
+}
+
 gboolean
 getClientID (DisplayInfo *display_info, Window window, gchar **client_id)
 {
@@ -987,7 +1036,7 @@ getClientID (DisplayInfo *display_info, Window window, gchar **client_id)
     *client_id = NULL;
     g_return_val_if_fail (window != None, FALSE);
 
-    if ((id = getClientLeader (display_info, window)))
+    if (getWindowProp (display_info, window, WM_CLIENT_LEADER, &id) && (id != None))
     {
         if (XGetTextProperty (display_info->dpy, id, &tp, display_info->atoms[SM_CLIENT_ID]))
         {
@@ -1015,7 +1064,7 @@ getWindowCommand (DisplayInfo *display_info, Window window, char ***argv, int *a
     {
         return TRUE;
     }
-    if ((id = getClientLeader (display_info, window)))
+    if (getWindowProp (display_info, window, WM_CLIENT_LEADER, &id) && (id != None))
     {
         if (XGetCommand (display_info->dpy, id, argv, argc) && (*argc > 0))
         {
@@ -1126,7 +1175,7 @@ gboolean
 setXAtomManagerOwner (DisplayInfo *display_info, Atom atom, Window root, Window w)
 {
     XClientMessageEvent ev;
-    Time server_time;
+    guint32 server_time;
     int status;
 
     g_return_val_if_fail (root != None, FALSE);
@@ -1167,55 +1216,40 @@ setAtomIdManagerOwner (DisplayInfo *display_info, int atom_id, Window root, Wind
     return setXAtomManagerOwner(display_info, display_info->atoms[atom_id], root, w);
 }
 
-
-static Bool
-checkPropEvent (Display *display, XEvent *xevent, XPointer arg)
+void
+updateXserverTime (DisplayInfo *display_info)
 {
-    DisplayInfo *display_info;
-    ScreenInfo *screen_info;
+    char c = '\0';
 
-    display_info = (DisplayInfo *) arg;
-    g_return_val_if_fail (display_info, FALSE);
+    g_return_if_fail (display_info);
 
-    screen_info = myDisplayGetDefaultScreen (display_info);
-    g_return_val_if_fail (screen_info, FALSE);
-
-    if ((xevent->type == PropertyNotify) &&
-        (xevent->xproperty.window == screen_info->xfwm4_win) &&
-        (xevent->xproperty.atom == display_info->atoms[XFWM4_TIMESTAMP_PROP]))
-    {
-        return TRUE;
-    }
-
-    return FALSE;
+    XChangeProperty (display_info->dpy, display_info->timestamp_win,
+                     display_info->atoms[XFWM4_TIMESTAMP_PROP],
+                     display_info->atoms[XFWM4_TIMESTAMP_PROP],
+                     8, PropModeReplace, (unsigned char *) &c, 1);
 }
 
-Time
+guint32
 getXServerTime (DisplayInfo *display_info)
 {
     ScreenInfo *screen_info;
     XEvent xevent;
-    Time timestamp;
-    char c = '\0';
+    guint32 timestamp;
 
-    g_return_val_if_fail (display_info, (Time) CurrentTime);
+    g_return_val_if_fail (display_info, CurrentTime);
     timestamp = myDisplayGetCurrentTime (display_info);
-    if (timestamp == (Time) CurrentTime)
+    if (timestamp == CurrentTime)
     {
         screen_info = myDisplayGetDefaultScreen (display_info);
-        g_return_val_if_fail (screen_info,  (Time) CurrentTime);
+        g_return_val_if_fail (screen_info,  CurrentTime);
 
         TRACE ("getXServerTime: Using X server roundtrip");
-        XChangeProperty (display_info->dpy, screen_info->xfwm4_win, 
-                         display_info->atoms[XFWM4_TIMESTAMP_PROP],
-                         display_info->atoms[XFWM4_TIMESTAMP_PROP],
-                         8, PropModeReplace, (unsigned char *) &c, 1);
-        XIfEvent (display_info->dpy, &xevent, checkPropEvent, (XPointer) display_info);
-
-        timestamp = (Time) myDisplayUpdateCurrentTime (display_info, &xevent);
+        updateXserverTime (display_info);
+        XWindowEvent (display_info->dpy, display_info->timestamp_win, PropertyChangeMask, &xevent);
+        timestamp = myDisplayUpdateCurrentTime (display_info, &xevent);
     }
 
-    TRACE ("getXServerTime gives timestamp=%u", (unsigned int) timestamp);
+    TRACE ("getXServerTime gives timestamp=%u", (guint32) timestamp);
     return timestamp;
 }
 
@@ -1324,44 +1358,3 @@ getWindowStartupId (DisplayInfo *display_info, Window w, gchar **startup_id)
     return FALSE;
 }
 #endif
-
-#ifdef HAVE_XSYNC
-gboolean
-getXSyncCounter (DisplayInfo *display_info, Window window, XSyncCounter *counter)
-{
-    long val;
-
-    g_return_val_if_fail (window != None, FALSE);
-    g_return_val_if_fail (counter != NULL, FALSE);
-    TRACE ("entering getXSyncCounter");
-
-    val = 0;
-    if (getHint (display_info, window, NET_WM_SYNC_REQUEST_COUNTER, &val))
-    {
-        *counter = (XSyncCounter) val;
-        return TRUE;
-    }
-
-    return FALSE;
-}
-
-void
-sendXSyncRequest (DisplayInfo *display_info, Window window, XSyncValue value)
-{
-    XClientMessageEvent xev;
-
-    g_return_if_fail (window != None);
-    TRACE ("entering getXSyncCounter");
-
-    xev.type = ClientMessage;
-    xev.window = window;
-    xev.message_type = display_info->atoms[WM_PROTOCOLS];
-    xev.format = 32;
-    xev.data.l[0] = (long) display_info->atoms[NET_WM_SYNC_REQUEST];
-    xev.data.l[1] = (long) myDisplayGetCurrentTime (display_info);
-    xev.data.l[2] = (long) XSyncValueLow32 (value);
-    xev.data.l[3] = (long) XSyncValueHigh32 (value);
-    xev.data.l[4] = (long) 0L;
-    XSendEvent (display_info->dpy, window, FALSE, NoEventMask, (XEvent *) &xev);
-}
-#endif /* HAVE_XSYNC */
