@@ -1,4 +1,4 @@
-/*      $Id: misc.c 23589 2006-11-01 21:14:46Z olivier $
+/*      $Id$
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,15 +12,17 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
+
 
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
+        xfwm4    - (c) 2002-2009 Olivier Fourdan
 
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <X11/Xlib.h>
@@ -104,7 +106,7 @@ createGC (ScreenInfo *screen_info, char *col, int func, XFontStruct * font,
 }
 
 void
-sendClientMessage (ScreenInfo *screen_info, Window w, int atom_id, Time timestamp)
+sendClientMessage (ScreenInfo *screen_info, Window w, int atom_id, guint32 timestamp)
 {
     DisplayInfo *display_info;
     XClientMessageEvent ev;
@@ -117,13 +119,14 @@ sendClientMessage (ScreenInfo *screen_info, Window w, int atom_id, Time timestam
     ev.window = w;
     ev.message_type = display_info->atoms[WM_PROTOCOLS];
     ev.format = 32;
+    ev.send_event = TRUE;
     ev.data.l[0] = display_info->atoms[atom_id];
     ev.data.l[1] = timestamp;
     XSendEvent (myScreenGetXDisplay (screen_info), w, FALSE, 0L, (XEvent *)&ev);
 }
 
 void
-sendRootMessage (ScreenInfo *screen_info, int atom_id, long value, Time timestamp)
+sendRootMessage (ScreenInfo *screen_info, int atom_id, long value, guint32 timestamp)
 {
     DisplayInfo *display_info;
     XClientMessageEvent ev;
@@ -232,63 +235,30 @@ placeSidewalks(ScreenInfo *screen_info, gboolean activate)
     }
 }
 
-/*
-   gdk_screen_get_monitor_at_point () doesn't give accurate results
-   when the point is off screen, use my own implementation from xfce 3
- */
-gint
-find_monitor_at_point (GdkScreen *screen, gint x, gint y)
+gchar*
+get_atom_name (DisplayInfo *display_info, Atom atom)
 {
-    static gint cache_monitor = -1;
-    static gint cache_x, cache_y;
-    gint dx, dy, center_x, center_y;
-    guint32 distsquare, min_distsquare;
-    gint num_monitors, nearest_monitor, i;
-    GdkRectangle monitor;
+    gchar *value;
+    gchar *xname;
 
-    g_return_val_if_fail (GDK_IS_SCREEN (screen), -1);
-
-    /* Cache system */
-    if ((cache_monitor >= 0) && (x == cache_x) && (y == cache_y))
+    if (atom)
     {
-        return (cache_monitor);
-    }
-
-    cache_x = x;
-    cache_y = y;
-
-    /* No monitor has been eligible, use the closest one */
-
-    min_distsquare = ((guint32) 0xffffffff);
-    nearest_monitor = 0;
-
-    num_monitors = gdk_screen_get_n_monitors (screen);
-    for (i = 0; i < num_monitors; i++)
-    {
-        gdk_screen_get_monitor_geometry (screen, i, &monitor);
-
-        if ((x >= monitor.x) && (x < monitor.x + monitor.width) &&
-            (y >= monitor.y) && (y < (monitor.y + monitor.height)))
+        xname = (gchar *) XGetAtomName (display_info->dpy, atom);
+        if (xname)
         {
-            cache_monitor = i;
-            return i;
+            value = g_strdup (xname);
+            XFree ((char *) xname);
         }
-
-        center_x = monitor.x + (monitor.width / 2);
-        center_y = monitor.y + (monitor.height / 2);
-
-        dx = x - center_x;
-        dy = y - center_y;
-
-        distsquare = (dx * dx) + (dy * dy);
-
-        if (distsquare < min_distsquare)
+        else
         {
-            min_distsquare = distsquare;
-            nearest_monitor = i;
+            value = g_strdup ("Unknown");
         }
     }
+    else
+    {
+        value = g_strdup ("None");
+    }
 
-    cache_monitor = nearest_monitor;
-    return (nearest_monitor);
+    return value;
 }
+
