@@ -1,4 +1,4 @@
-/*      $Id: mywindow.h 23704 2006-11-04 22:36:42Z olivier $
+/*      $Id$
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,10 +12,12 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
+
 
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
+        xfwm4    - (c) 2002-2009 Olivier Fourdan
 
  */
 
@@ -23,7 +25,7 @@
 #define INC_MYWINDOW_H
 
 #ifdef HAVE_CONFIG_H
-#  include "config.h"
+#include "config.h"
 #endif
 
 #include <X11/Xlib.h>
@@ -58,6 +60,7 @@ void                     xfwmWindowCreate                       (ScreenInfo *,
                                                                  gint,
                                                                  Window,
                                                                  xfwmWindow *,
+                                                                 long,
                                                                  Cursor);
 void                     xfwmWindowTemp                         (ScreenInfo *,
                                                                  Visual *,
@@ -80,7 +83,7 @@ void                     xfwmWindowShow                         (xfwmWindow *,
 void                     xfwmWindowHide                         (xfwmWindow *);
 gboolean                 xfwmWindowVisible                      (xfwmWindow *);
 gboolean                 xfwmWindowDeleted                      (xfwmWindow *);
-void                     xfwmWindowSetBG                        (xfwmWindow *, 
+void                     xfwmWindowSetBG                        (xfwmWindow *,
                                                                  xfwmPixmap *);
 
 #endif /* INC_MYWINDOW_H */
