@@ -1,26 +1,28 @@
-/*      $Id: parserc.c,v 1.33 2004/09/21 19:27:01 olivier Exp $
- 
+/*      $Id$
+
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
         any later version.
- 
+
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
         GNU General Public License for more details.
- 
+
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- 
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
+
+
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2004 Olivier Fourdan
- 
+        xfwm4    - (c) 2002-2009 Olivier Fourdan
+
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <X11/Xlib.h>
@@ -28,7 +30,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <glib.h>
-#include <libxfce4util/libxfce4util.h> 
+#include <libxfce4util/libxfce4util.h>
 
 #include "settings.h"
 #include "parserc.h"
@@ -37,8 +39,10 @@
 #define DEFAULT_THEME "Default"
 #endif
 
+#define TOINT(x)                (x ? atoi(x) : 0)
+
 gboolean
-parseRc (const gchar * file, const gchar * dir, Settings rc[])
+parseRc (const gchar * file, const gchar * dir, Settings *rc)
 {
     gchar buf[255];
     gchar *filename, *lvalue, *rvalue;
@@ -78,13 +82,14 @@ parseRc (const gchar * file, const gchar * dir, Settings rc[])
 }
 
 gboolean
-checkRc (Settings rc[])
+checkRc (Settings *rc)
 {
     gint i;
-    gboolean rval = TRUE;
+    gboolean rval;
 
     TRACE ("entering checkRc");
 
+    rval = TRUE;
     for (i = 0; rc[i].option; i++)
     {
         if (rc[i].required && !rc[i].value)
@@ -96,8 +101,8 @@ checkRc (Settings rc[])
     return rval;
 }
 
-gchar *
-getValue (const gchar * option, Settings rc[])
+GValue *
+getGValue (const gchar * option, Settings *rc)
 {
     gint i;
 
@@ -116,7 +121,42 @@ getValue (const gchar * option, Settings rc[])
 }
 
 gboolean
-setValue (const gchar * lvalue, const gchar * rvalue, Settings rc[])
+setValue (const gchar * lvalue, const gchar *rvalue, Settings *rc)
+{
+    gint i;
+
+    TRACE ("entering setValue");
+
+    g_return_val_if_fail (lvalue != NULL, FALSE);
+    g_return_val_if_fail (rvalue != NULL, FALSE);
+
+    for (i = 0; rc[i].option; i++)
+    {
+        if (!g_ascii_strcasecmp (lvalue, rc[i].option))
+        {
+            if (rvalue)
+            {
+                switch (rc[i].type)
+                {
+                    case G_TYPE_STRING:
+                        setStringValue (lvalue, rvalue, rc);
+                        break;
+                    case G_TYPE_INT:
+                        setIntValue (lvalue, TOINT(rvalue), rc);
+                        break;
+                    case G_TYPE_BOOLEAN:
+                        setBooleanValue (lvalue, !g_ascii_strcasecmp ("true", rvalue), rc);
+                        break;
+                }
+                return TRUE;
+            }
+        }
+    }
+    return FALSE;
+}
+
+gboolean
+setGValue (const gchar * lvalue, const GValue *rvalue, Settings *rc)
 {
     gint i;
 
@@ -133,10 +173,16 @@ setValue (const gchar * lvalue, const gchar * rvalue, Settings rc[])
             {
                 if (rc[i].value)
                 {
-                    g_free (rc[i].value);
+                    g_value_unset (rc[i].value);
+                    g_value_init (rc[i].value, G_VALUE_TYPE(rvalue));
                 }
-                rc[i].value = g_strdup (rvalue);
-                TRACE ("%s=%s", rc[i].option, rc[i].value);
+                else
+                {
+                    rc[i].value = g_new0(GValue, 1);
+                    g_value_init (rc[i].value, G_VALUE_TYPE(rvalue));
+                }
+
+                g_value_copy (rvalue, rc[i].value);
                 return TRUE;
             }
         }
@@ -145,26 +191,33 @@ setValue (const gchar * lvalue, const gchar * rvalue, Settings rc[])
 }
 
 gboolean
-setBooleanValueFromInt (const gchar * lvalue, int value, Settings rc[])
+setBooleanValue (const gchar * lvalue, gboolean value, Settings *rc)
 {
-    return setValue (lvalue, value ? "true" : "false", rc);
+    GValue tmp_val = {0, };
+    g_value_init(&tmp_val, G_TYPE_BOOLEAN);
+    g_value_set_boolean(&tmp_val, value);
+    return setGValue (lvalue, &tmp_val, rc);
 }
 
 gboolean
-setIntValueFromInt (const gchar * lvalue, int value, Settings rc[])
+setIntValue (const gchar * lvalue, gint value, Settings *rc)
 {
-    gchar *s;
-    gboolean result;
+    GValue tmp_val = {0, };
+    g_value_init(&tmp_val, G_TYPE_INT);
+    g_value_set_int(&tmp_val, value);
+    return setGValue (lvalue, &tmp_val, rc);
+}
 
-    s = g_strdup_printf ("%i", value);
-    result = setValue (lvalue, s, rc);
-    g_free (s);
-
-    return result;
+gboolean
+setStringValue (const gchar * lvalue, const gchar *value, Settings *rc)
+{
+    GValue tmp_val = {0, };
+    g_value_init(&tmp_val, G_TYPE_STRING);
+    g_value_set_static_string(&tmp_val, value);
+    return setGValue (lvalue, &tmp_val, rc);
 }
 
 gchar *
-
 getSystemThemeDir (void)
 {
     return g_build_filename (DATADIR, "themes", DEFAULT_THEME, "xfwm4", NULL);
@@ -175,7 +228,7 @@ getThemeDir (const gchar * theme, const gchar * file)
 {
     if (!theme)
     {
-        return g_build_filename (DATADIR, "themes", DEFAULT_THEME, "xfwm4", 
+        return g_build_filename (DATADIR, "themes", DEFAULT_THEME, "xfwm4",
                                  NULL);
     }
     else if (g_path_is_absolute (theme))
@@ -192,16 +245,16 @@ getThemeDir (const gchar * theme, const gchar * file)
     else
     {
         gchar *test_file, *path;
-        
+
         path = g_build_filename (theme, "xfwm4", file, NULL);
 
-        xfce_resource_push_path (XFCE_RESOURCE_THEMES, 
+        xfce_resource_push_path (XFCE_RESOURCE_THEMES,
                                  DATADIR G_DIR_SEPARATOR_S "themes");
         test_file = xfce_resource_lookup (XFCE_RESOURCE_THEMES, path);
         xfce_resource_pop_path (XFCE_RESOURCE_THEMES);
 
         g_free (path);
- 
+
         if (test_file)
         {
             path = g_path_get_dirname (test_file);
@@ -209,13 +262,13 @@ getThemeDir (const gchar * theme, const gchar * file)
             return path;
         }
     }
-    
+
     /* Pfew, really can't find that theme nowhere! */
     return getSystemThemeDir ();
 }
 
 void
-freeRc (Settings rc[])
+freeRc (Settings *rc)
 {
     gint i;
 
@@ -225,8 +278,75 @@ freeRc (Settings rc[])
     {
         if (rc[i].value)
         {
+            g_value_unset(rc[i].value);
             g_free (rc[i].value);
             rc[i].value = NULL;
         }
     }
 }
+
+const gchar *
+getStringValue (const gchar *option, Settings *rc)
+{
+    gint i;
+
+    TRACE ("entering getStringValue");
+
+    g_return_val_if_fail (option != NULL, NULL);
+
+    for (i = 0; rc[i].option; i++)
+    {
+        if (!g_ascii_strcasecmp (option, rc[i].option))
+        {
+            if (rc[i].value == NULL)
+                return NULL;
+            g_return_val_if_fail(G_VALUE_TYPE(rc[i].value) == G_TYPE_STRING, NULL);
+            return g_value_get_string(rc[i].value);
+        }
+    }
+    return NULL;
+}
+
+gint
+getIntValue (const gchar *option, Settings *rc)
+{
+    gint i;
+
+    TRACE ("entering getIntValue");
+
+    g_return_val_if_fail (option != NULL, 0);
+
+    for (i = 0; rc[i].option; i++)
+    {
+        if (!g_ascii_strcasecmp (option, rc[i].option))
+        {
+            if (rc[i].value == NULL)
+                return 0;
+            g_return_val_if_fail(G_VALUE_TYPE(rc[i].value) == G_TYPE_INT, 0);
+            return g_value_get_int(rc[i].value);
+        }
+    }
+    return 0;
+}
+
+gboolean
+getBoolValue (const gchar *option, Settings *rc)
+{
+    gint i;
+
+    TRACE ("entering getBoolValue");
+
+    g_return_val_if_fail (option != NULL, FALSE);
+
+    for (i = 0; rc[i].option; i++)
+    {
+        if (!g_ascii_strcasecmp (option, rc[i].option))
+        {
+            if (rc[i].value == NULL)
+                return FALSE;
+            g_return_val_if_fail(G_VALUE_TYPE(rc[i].value) == G_TYPE_BOOLEAN, FALSE);
+            return g_value_get_boolean(rc[i].value);
+        }
+    }
+    return FALSE;
+}
