@@ -1,4 +1,4 @@
-/*      $Id: placement.c 23704 2006-11-04 22:36:42Z olivier $
+/*      $Id$
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,14 +12,16 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
 
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
+
+        xfwm4    - (c) 2002-2009 Olivier Fourdan
 
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <X11/Xlib.h>
@@ -36,10 +38,33 @@
 #include "frame.h"
 #include "netwm.h"
 
-static unsigned long overlapX (int x0, int x1, int tx0, int tx1);
-static unsigned long overlapY (int y0, int y1, int ty0, int ty1);
-static unsigned long overlap (int x0, int y0, int x1, int y1,
-                              int tx0, int ty0, int tx1, int ty1);
+/* Compute rectangle overlap area */
+
+static unsigned long
+segment_overlap (int x0, int x1, int tx0, int tx1)
+{
+    if (tx0 > x0)
+    {
+        x0 = tx0;
+    }
+    if (tx1 < x1)
+    {
+        x1 = tx1;
+    }
+    if (x1 <= x0)
+    {
+        return 0;
+    }
+    return (x1 - x0);
+}
+
+static unsigned long
+overlap (int x0, int y0, int x1, int y1, int tx0, int ty0, int tx1, int ty1)
+{
+    /* Compute overlapping box */
+    return (segment_overlap (x0, x1, tx0, tx1)
+            * segment_overlap (y0, y1, ty0, ty1));
+}
 
 static unsigned long
 clientStrutAreaOverlap (int x, int y, int w, int h, Client * c)
@@ -70,51 +95,6 @@ clientStrutAreaOverlap (int x, int y, int w, int h, Client * c)
     return sigma;
 }
 
-/* Compute rectangle overlap area */
-
-static unsigned long
-overlapX (int x0, int x1, int tx0, int tx1)
-{
-    if (tx0 > x0)
-    {
-        x0 = tx0;
-    }
-    if (tx1 < x1)
-    {
-        x1 = tx1;
-    }
-    if (x1 <= x0)
-    {
-        return 0;
-    }
-    return (x1 - x0);
-}
-
-static unsigned long
-overlapY (int y0, int y1, int ty0, int ty1)
-{
-    if (ty0 > y0)
-    {
-        y0 = ty0;
-    }
-    if (ty1 < y1)
-    {
-        y1 = ty1;
-    }
-    if (y1 <= y0)
-    {
-        return 0;
-    }
-    return (y1 - y0);
-}
-
-static unsigned long
-overlap (int x0, int y0, int x1, int y1, int tx0, int ty0, int tx1, int ty1)
-{
-    /* Compute overlapping box */
-    return (overlapX (x0, x1, tx0, tx1) * overlapY (y0, y1, ty0, ty1));
-}
-
 void
 clientMaxSpace (ScreenInfo *screen_info, int *x, int *y, int *w, int *h)
 {
@@ -193,7 +173,7 @@ clientCkeckTitle (Client * c)
     screen_info = c->screen_info;
     for (c2 = screen_info->clients, i = 0; i < screen_info->client_count; c2 = c2->next, i++)
     {
-        if ((c2 != c) && clientStrutAreaOverlap(frame_x, frame_y, frame_width, frame_top, c2))
+        if ((c2 != c) && clientStrutAreaOverlap (frame_x, frame_y, frame_width, frame_top, c2))
         {
             return FALSE;
         }
@@ -222,7 +202,6 @@ clientConstrainPos (Client * c, gboolean show_full)
     int screen_width, screen_height;
     unsigned int ret;
     GdkRectangle rect;
-    gint monitor_nbr;
     gint min_visible;
 
     g_return_val_if_fail (c != NULL, 0);
@@ -245,8 +224,7 @@ clientConstrainPos (Client * c, gboolean show_full)
     cy = frame_y + (frame_height / 2);
 
     screen_info = c->screen_info;
-    monitor_nbr = find_monitor_at_point (screen_info->gscr, cx, cy);
-    gdk_screen_get_monitor_geometry (screen_info->gscr, monitor_nbr, &rect);
+    myScreenFindMonitorAtPoint (screen_info, cx, cy, &rect);
 
     screen_width = screen_info->width;
     screen_height = screen_info->height;
@@ -273,10 +251,10 @@ clientConstrainPos (Client * c, gboolean show_full)
                 && (c2 != c))
             {
                 /* Right */
-                if (overlapY (frame_y, frame_y + frame_height,
+                if (segment_overlap (frame_y, frame_y + frame_height,
                               c2->struts[STRUTS_RIGHT_START_Y], c2->struts[STRUTS_RIGHT_END_Y]))
                 {
-                    if (overlapX (frame_x, frame_x + frame_width,
+                    if (segment_overlap (frame_x, frame_x + frame_width,
                                   screen_width - c2->struts[STRUTS_RIGHT],
                                   screen_width))
                     {
@@ -287,10 +265,10 @@ clientConstrainPos (Client * c, gboolean show_full)
                 }
 
                 /* Bottom */
-                if (overlapX (frame_x, frame_x + frame_width,
+                if (segment_overlap (frame_x, frame_x + frame_width,
                               c2->struts[STRUTS_BOTTOM_START_X], c2->struts[STRUTS_BOTTOM_END_X]))
                 {
-                    if (overlapY (frame_y, frame_y + frame_height,
+                    if (segment_overlap (frame_y, frame_y + frame_height,
                                   screen_height - c2->struts[STRUTS_BOTTOM],
                                   screen_height))
                     {
@@ -335,10 +313,10 @@ clientConstrainPos (Client * c, gboolean show_full)
                 && (c2 != c))
             {
                 /* Left */
-                if (overlapY (frame_y, frame_y + frame_height,
+                if (segment_overlap (frame_y, frame_y + frame_height,
                               c2->struts[STRUTS_LEFT_START_Y], c2->struts[STRUTS_LEFT_END_Y]))
                 {
-                    if (overlapX (frame_x, frame_x + frame_width,
+                    if (segment_overlap (frame_x, frame_x + frame_width,
                                   0, c2->struts[STRUTS_LEFT]))
                     {
                         c->x = c2->struts[STRUTS_LEFT] + frame_left;
@@ -348,12 +326,12 @@ clientConstrainPos (Client * c, gboolean show_full)
                 }
 
                 /* Top */
-                if (overlapX (frame_x,
+                if (segment_overlap (frame_x,
                               frame_x + frame_width,
                               c2->struts[STRUTS_TOP_START_X],
                               c2->struts[STRUTS_TOP_END_X]))
                 {
-                    if (overlapY (frame_y, frame_y + frame_height,
+                    if (segment_overlap (frame_y, frame_y + frame_height,
                                   0, c2->struts[STRUTS_TOP]))
                     {
                         c->y = c2->struts[STRUTS_TOP] + frame_top;
@@ -404,7 +382,7 @@ clientConstrainPos (Client * c, gboolean show_full)
                 && (c2 != c))
             {
                 /* Right */
-                if (overlapY (frame_y, frame_y + frame_height,
+                if (segment_overlap (frame_y, frame_y + frame_height,
                               c2->struts[STRUTS_RIGHT_START_Y], c2->struts[STRUTS_RIGHT_END_Y]))
                 {
                     if (frame_x >= screen_width - c2->struts[STRUTS_RIGHT] - min_visible)
@@ -416,7 +394,7 @@ clientConstrainPos (Client * c, gboolean show_full)
                 }
 
                 /* Left */
-                if (overlapY (frame_y, frame_y + frame_height,
+                if (segment_overlap (frame_y, frame_y + frame_height,
                               c2->struts[STRUTS_LEFT_START_Y], c2->struts[STRUTS_LEFT_END_Y]))
                 {
                     if (frame_x + frame_width <= c2->struts[STRUTS_LEFT] + min_visible)
@@ -428,7 +406,7 @@ clientConstrainPos (Client * c, gboolean show_full)
                 }
 
                 /* Bottom */
-                if (overlapX (frame_x, frame_x + frame_width,
+                if (segment_overlap (frame_x, frame_x + frame_width,
                               c2->struts[STRUTS_BOTTOM_START_X], c2->struts[STRUTS_BOTTOM_END_X]))
                 {
                     if (frame_y >= screen_height - c2->struts[STRUTS_BOTTOM] - min_visible)
@@ -440,10 +418,10 @@ clientConstrainPos (Client * c, gboolean show_full)
                 }
 
                 /* Top */
-                if (overlapX (frame_x, frame_x + frame_width,
+                if (segment_overlap (frame_x, frame_x + frame_width,
                               c2->struts[STRUTS_TOP_START_X], c2->struts[STRUTS_TOP_END_X]))
                 {
-                    if (overlapY (frame_y, frame_y + frame_visible, 0, c2->struts[STRUTS_TOP]))
+                    if (segment_overlap (frame_y, frame_y + frame_visible, 0, c2->struts[STRUTS_TOP]))
                     {
                         c->y = c2->struts[STRUTS_TOP] + frame_top;
                         frame_y = frameY (c);
@@ -467,11 +445,15 @@ clientConstrainPos (Client * c, gboolean show_full)
    translation in Xinerama to center window on physical screen
    Not to be confused with clientConstrainPos()
  */
-void
-clientKeepVisible (Client * c)
+static void
+clientKeepVisible (Client * c, gint n_monitors, GdkRectangle *monitor_rect)
 {
+    ScreenInfo *screen_info;
+    GdkRectangle rect;
+    gboolean centered;
     int cx, cy;
     int diff_x, diff_y;
+    int monitor_nbr;
 
     g_return_if_fail (c != NULL);
     TRACE ("entering clientKeepVisible");
@@ -479,29 +461,40 @@ clientKeepVisible (Client * c)
 
     cx = frameX (c) + (frameWidth (c) / 2);
     cy = frameY (c) + (frameHeight (c) / 2);
+    screen_info = c->screen_info;
 
-    /* Translate coodinates to center on physical screen */
-
-    diff_x = abs (c->size->x - ((c->screen_info->width - c->width) / 2));
-    diff_y = abs (c->size->y - ((c->screen_info->height - c->height) / 2));
-
-    if (((gdk_screen_get_n_monitors (c->screen_info->gscr) > 1) && (diff_x < 25) && (diff_y < 25)) ||
-        ((frameX (c) == 0) && (frameY (c) == 0) && (c->type & (WINDOW_TYPE_DIALOG)) && !clientIsTransient (c)))
+    centered = FALSE;
+    if ((c->size->x == 0) && (c->size->y == 0) && (c->type & (WINDOW_TYPE_DIALOG)))
     {
-        GdkRectangle rect;
-        gint monitor_nbr;
+        /* Dialogs that place temselves in (0,0) will be centered */
+        centered = TRUE;
+    }
+    else if (n_monitors > 1)
+    {
+        /* First, check if the window is centered on the whole screen */
+        diff_x = abs (c->size->x - ((c->screen_info->width - c->size->width) / 2));
+        diff_y = abs (c->size->y - ((c->screen_info->height - c->size->height) / 2));
 
+        monitor_nbr = 0;
+        centered = ((diff_x < 25) && (diff_y < 25));
+
+        while ((!centered) && (monitor_nbr < n_monitors))
+        {
+            gdk_screen_get_monitor_geometry (screen_info->gscr, monitor_nbr, &rect);
+            diff_x = abs (c->size->x - ((rect.width - c->size->width) / 2));
+            diff_y = abs (c->size->y - ((rect.height - c->size->height) / 2));
+            centered = ((diff_x < 25) && (diff_y < 25));
+            monitor_nbr++;
+        }
+    }
+    if (centered)
+    {
         /* We consider that the windows is centered on screen,
          * Thus, will move it so its center on the current
          * physical screen
          */
-        getMouseXY (c->screen_info, c->screen_info->xroot, &cx, &cy);
-
-        monitor_nbr = find_monitor_at_point (c->screen_info->gscr, cx, cy);
-        gdk_screen_get_monitor_geometry (c->screen_info->gscr, monitor_nbr, &rect);
-
-        c->x = rect.x + (rect.width - c->width) / 2;
-        c->y = rect.y + (rect.height - c->height) / 2;
+        c->x = monitor_rect->x + (monitor_rect->width - c->width) / 2;
+        c->y = monitor_rect->y + (monitor_rect->height - c->height) / 2;
     }
     clientConstrainPos (c, TRUE);
 }
@@ -510,21 +503,31 @@ static void
 clientAutoMaximize (Client * c, int full_w, int full_h)
 {
     if (FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN) ||
-        FLAG_TEST (c->xfwm_flags, XFWM_FLAG_LEGACY_FULLSCREEN))
+        FLAG_TEST (c->xfwm_flags, XFWM_FLAG_LEGACY_FULLSCREEN) ||
+        !FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER))
     {
-        /* Fullscree nwindows should not be maximized */
+        /*
+         * Fullscreen or undecorated windows should not be
+         * automatically maximized...
+         */
         return;
     }
 
     if (!FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED_HORIZ) &&
         (frameWidth (c) > full_w))
     {
+        TRACE ("The application \"%s\" has requested a window width "
+               "(%u) larger than the actual width available in the workspace (%u), "
+               "the window will be maximized horizontally.", c->name, frameWidth (c), full_w);
         FLAG_SET (c->flags, CLIENT_FLAG_MAXIMIZED_HORIZ);
     }
 
     if (!FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED_VERT) &&
         (frameHeight (c) > full_h))
     {
+        TRACE ("The application \"%s\" has requested a window height "
+               "(%u) larger than the actual height available in the workspace (%u), "
+               "the window will be maximized vertically.", c->name, frameHeight (c), full_h);
         FLAG_SET (c->flags, CLIENT_FLAG_MAXIMIZED_VERT);
     }
 }
@@ -559,11 +562,11 @@ smartPlacement (Client * c, int full_x, int full_y, int full_w, int full_h)
     best_x = full_x + frameLeft (c);
     best_y = full_y + frameTop (c);
 
-    for (test_y = full_y + frameTop (c); test_y <= ymax;
-        test_y += 8)
+    test_y = full_y + frameTop (c);
+    do
     {
-        for (test_x = full_x + frameLeft (c);
-            test_x <= xmax; test_x += 8)
+        test_x = full_x + frameLeft (c);
+        do
         {
             gfloat count_overlaps = 0.0;
             TRACE ("analyzing %i clients", screen_info->client_count);
@@ -601,8 +604,13 @@ smartPlacement (Client * c, int full_x, int full_y, int full_w, int full_h)
             {
                 first = FALSE;
             }
+            test_x += 8;
         }
+        while (test_x <= xmax);
+        test_y += 8;
     }
+    while (test_y <= ymax);
+
     c->x = best_x;
     c->y = best_y;
 }
@@ -617,6 +625,22 @@ centerPlacement (Client * c, int full_x, int full_y, int full_w, int full_h)
     c->y = MAX (full_y + frameTop(c) + (full_h - frameHeight(c)) / 2, full_y + frameTop(c));
 }
 
+static void
+mousePlacement (Client * c, int full_x, int full_y, int full_w, int full_h, int mx, int my)
+{
+    g_return_if_fail (c != NULL);
+    TRACE ("entering centerPlacement");
+
+    c->x = mx + frameLeft(c) - frameWidth(c) / 2;
+    c->y = my + frameTop(c) - frameHeight(c) / 2;
+
+    c->x = MIN (c->x, full_x + full_w - frameWidth(c) + frameLeft(c));
+    c->y = MIN (c->y, full_y + full_h - frameHeight(c) + frameTop(c));
+
+    c->x = MAX (c->x, full_x + frameLeft(c));
+    c->y = MAX (c->y, full_y + frameTop(c));
+}
+
 void
 clientInitPosition (Client * c)
 {
@@ -624,53 +648,54 @@ clientInitPosition (Client * c)
     Client *c2;
     GdkRectangle rect;
     int full_x, full_y, full_w, full_h, msx, msy;
-    gint monitor_nbr;
-    gboolean place = TRUE;
+    gint n_monitors;
+    gboolean place;
+    gboolean position;
 
     g_return_if_fail (c != NULL);
     TRACE ("entering clientInitPosition");
 
-    clientGravitate (c, APPLY);
-
     screen_info = c->screen_info;
     msx = 0;
     msy = 0;
+    position = (c->size->flags & (PPosition | USPosition));
 
-    if ((c->size->flags & (PPosition | USPosition)) ||
-        (c->type & (WINDOW_TYPE_DONT_PLACE)) ||
-        ((c->type & (WINDOW_TYPE_DIALOG)) && !clientIsTransient (c)))
+    n_monitors = gdk_screen_get_n_monitors (c->screen_info->gscr);
+    if ((n_monitors > 1) || (screen_info->params->placement_mode == PLACE_MOUSE))
     {
-        if (CONSTRAINED_WINDOW (c))
-        {
-            clientKeepVisible (c);
-        }
-        msx = frameX (c) + (frameWidth (c) / 2);
-        msy = frameY (c) + (frameHeight (c) / 2);
-        place = FALSE;
+        getMouseXY (screen_info, screen_info->xroot, &msx, &msy);
+        myScreenFindMonitorAtPoint (screen_info, msx, msy, &rect);
     }
-    else if (clientIsTransient (c) && (c2 = clientGetTransient (c)))
+    else
     {
+        gdk_screen_get_monitor_geometry (screen_info->gscr, 0, &rect);
+    }
+    if (position || (c->type & (WINDOW_TYPE_DONT_PLACE | WINDOW_TYPE_DIALOG)) || clientIsTransient (c))
+    {
+        if (!position && clientIsTransient (c) && (c2 = clientGetTransient (c)))
+        {
+            /* Center transient relative to their parent window */
+            c->x = c2->x + (c2->width - c->width) / 2;
+            c->y = c2->y + (c2->height - c->height) / 2;
 
-        /* Center transient relative to their parent window */
-        c->x = c2->x + (c2->width - c->width) / 2;
-        c->y = c2->y + (c2->height - c->height) / 2;
+            if (n_monitors > 1)
+            {
+                msx = frameX (c) + (frameWidth (c) / 2);
+                msy = frameY (c) + (frameHeight (c) / 2);
+                myScreenFindMonitorAtPoint (screen_info, msx, msy, &rect);
+            }
+        }
         if (CONSTRAINED_WINDOW (c))
         {
-            clientKeepVisible (c);
+            clientKeepVisible (c, n_monitors, &rect);
         }
-        msx = frameX (c) + (frameWidth (c) / 2);
-        msy = frameY (c) + (frameHeight (c) / 2);
         place = FALSE;
     }
     else
     {
-        getMouseXY (screen_info, screen_info->xroot, &msx, &msy);
         place = TRUE;
     }
 
-    monitor_nbr = find_monitor_at_point (screen_info->gscr, msx, msy);
-    gdk_screen_get_monitor_geometry (screen_info->gscr, monitor_nbr, &rect);
-
     full_x = MAX (screen_info->params->xfwm_margins[STRUTS_LEFT], rect.x);
     full_y = MAX (screen_info->params->xfwm_margins[STRUTS_TOP], rect.y);
     full_w = MIN (screen_info->width - screen_info->params->xfwm_margins[STRUTS_RIGHT],
@@ -689,9 +714,19 @@ clientInitPosition (Client * c)
      */
     if (place)
     {
-        if ((screen_info->params->placement_ratio > 100) ||
-            ((frameWidth(c) >= full_w) && (frameHeight(c) >= full_h)) ||
+        if ((screen_info->params->placement_ratio >= 100) ||
             (100 * frameWidth(c) * frameHeight(c)) < (screen_info->params->placement_ratio * full_w * full_h))
+        {
+            if (screen_info->params->placement_mode == PLACE_MOUSE)
+            {
+                mousePlacement (c, full_x, full_y, full_w, full_h, msx, msy);
+            }
+            else
+            {
+                centerPlacement (c, full_x, full_y, full_w, full_h);
+            }
+        }
+        else if ((frameWidth(c) >= full_w) && (frameHeight(c) >= full_h))
         {
             centerPlacement (c, full_x, full_y, full_w, full_h);
         }
@@ -701,5 +736,215 @@ clientInitPosition (Client * c)
         }
     }
 
-    clientAutoMaximize (c, full_w, full_h);
+    if (c->type & WINDOW_REGULAR_FOCUSABLE)
+    {
+        clientAutoMaximize (c, full_w, full_h);
+    }
 }
+
+
+void
+clientFill (Client * c, int fill_type)
+{
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
+    Client *east_neighbour;
+    Client *west_neighbour;
+    Client *north_neighbour;
+    Client *south_neighbour;
+    Client *c2;
+    GdkRectangle rect;
+    XWindowChanges wc;
+    unsigned short mask;
+    int i, cx, cy, full_x, full_y, full_w, full_h;
+    int tmp_x, tmp_y, tmp_w, tmp_h;
+
+    g_return_if_fail (c != NULL);
+    TRACE ("entering clientFill");
+
+    if (!CLIENT_CAN_FILL_WINDOW (c))
+    {
+        return;
+    }
+
+    screen_info = c->screen_info;
+    display_info = screen_info->display_info;
+    mask = 0;
+    east_neighbour = NULL;
+    west_neighbour = NULL;
+    north_neighbour = NULL;
+    south_neighbour = NULL;
+
+    for (c2 = screen_info->clients, i = 0; i < screen_info->client_count; c2 = c2->next, i++)
+    {
+
+        /* Filter out all windows which are not visible, or not on the same layer
+         * as well as the client window itself
+         */
+        if ((c != c2) && FLAG_TEST (c2->xfwm_flags, XFWM_FLAG_VISIBLE) && (c2->win_layer == c->win_layer))
+        {
+            /* Fill horizontally */
+            if (fill_type & CLIENT_FILL_HORIZ)
+            {
+                /*
+                 * check if the neigbour client (c2) is located
+                 * east or west of our client.
+                 */
+                if (segment_overlap (frameY(c), frameY(c) + frameHeight(c), frameY(c2), frameY(c2) + frameHeight(c2)))
+                {
+                    if ((frameX(c2) + frameWidth(c2)) <= frameX(c))
+                    {
+                        if (west_neighbour)
+                        {
+                            /* Check if c2 is closer to the client
+                             * then the west neighbour already found
+                             */
+                            if ((frameX(west_neighbour) + frameWidth(west_neighbour)) < (frameX(c2) + frameWidth(c2)))
+                            {
+                                west_neighbour = c2;
+                            }
+                        }
+                        else
+                        {
+                            west_neighbour = c2;
+                        }
+                    }
+                    if ((frameX(c) + frameWidth(c)) <= frameX(c2))
+                    {
+                        /* Check if c2 is closer to the client
+                         * then the west neighbour already found
+                         */
+                        if (east_neighbour)
+                        {
+                            if (frameX(c2) < frameX(east_neighbour))
+                            {
+                                east_neighbour = c2;
+                            }
+                        }
+                        else
+                        {
+                            east_neighbour = c2;
+                        }
+                    }
+                }
+            }
+
+            /* Fill vertically */
+            if (fill_type & CLIENT_FILL_VERT)
+            {
+                /* check if the neigbour client (c2) is located
+                 * north or south of our client.
+                 */
+                if (segment_overlap (frameX(c), frameX(c) + frameWidth(c), frameX(c2), frameX(c2) + frameWidth(c2)))
+                {
+                    if ((frameY(c2) + frameHeight(c2)) <= frameY(c))
+                    {
+                        if (north_neighbour)
+                        {
+                            /* Check if c2 is closer to the client
+                             * then the north neighbour already found
+                             */
+                            if ((frameY(north_neighbour) + frameHeight(north_neighbour)) < (frameY(c2) + frameHeight(c2)))
+                            {
+                                north_neighbour = c2;
+                            }
+                        }
+                        else
+                        {
+                            north_neighbour = c2;
+                        }
+                    }
+                    if ((frameY(c) + frameHeight(c)) <= frameY(c2))
+                    {
+                        if (south_neighbour)
+                        {
+                            /* Check if c2 is closer to the client
+                             * then the south neighbour already found
+                             */
+                            if (frameY(c2) < frameY(south_neighbour))
+                            {
+                                south_neighbour = c2;
+                            }
+                        }
+                        else
+                        {
+                            south_neighbour = c2;
+                        }
+                    }
+                }
+            }
+        }
+    }
+
+    /* Compute the largest size available, based on struts, margins and Xinerama layout */
+    tmp_x = frameX (c);
+    tmp_y = frameY (c);
+    tmp_h = frameHeight (c);
+    tmp_w = frameWidth (c);
+
+    cx = tmp_x + (tmp_w / 2);
+    cy = tmp_y + (tmp_h / 2);
+
+    myScreenFindMonitorAtPoint (screen_info, cx, cy, &rect);
+
+    full_x = MAX (screen_info->params->xfwm_margins[STRUTS_LEFT], rect.x);
+    full_y = MAX (screen_info->params->xfwm_margins[STRUTS_TOP], rect.y);
+    full_w = MIN (screen_info->width - screen_info->params->xfwm_margins[STRUTS_RIGHT],
+                  rect.x + rect.width) - full_x;
+    full_h = MIN (screen_info->height - screen_info->params->xfwm_margins[STRUTS_BOTTOM],
+                  rect.y + rect.height) - full_y;
+
+    if ((fill_type & CLIENT_FILL) == CLIENT_FILL)
+    {
+        mask = CWX | CWY | CWHeight | CWWidth;
+        /* Adjust size to the largest size available, not covering struts */
+        clientMaxSpace (screen_info, &full_x, &full_y, &full_w, &full_h);
+    }
+    else if (fill_type & CLIENT_FILL_VERT)
+    {
+        mask = CWY | CWHeight;
+        /* Adjust size to the tallest size available, for the current horizontal position/width */
+        clientMaxSpace (screen_info, &tmp_x, &full_y, &tmp_w, &full_h);
+    }
+    else if (fill_type & CLIENT_FILL_HORIZ)
+    {
+        mask = CWX | CWWidth;
+        /* Adjust size to the widest size available, for the current vertical position/height */
+        clientMaxSpace (screen_info, &full_x, &tmp_y, &full_w, &tmp_h);
+    }
+
+    /* If there are neighbours, resize to their borders.
+     * If not, resize to the largest size available that you just have computed.
+     */
+
+    wc.x = full_x + frameLeft(c);
+    if (west_neighbour)
+    {
+        wc.x += MAX (frameX(west_neighbour) + frameWidth(west_neighbour) - full_x, 0);
+    }
+
+    wc.width = full_w - frameRight(c) - (wc.x - full_x);
+    if (east_neighbour)
+    {
+        wc.width -= MAX (full_w - (frameX(east_neighbour) - full_x), 0);
+    }
+
+    wc.y = full_y + frameTop(c);
+    if (north_neighbour)
+    {
+        wc.y += MAX (frameY(north_neighbour) + frameHeight(north_neighbour) - full_y, 0);
+    }
+
+    wc.height = full_h - frameBottom(c) - (wc.y - full_y);
+    if (south_neighbour)
+    {
+        wc.height -= MAX (full_h - (frameY(south_neighbour) - full_y), 0);
+    }
+
+    TRACE ("Fill size request: (%d,%d) %dx%d", wc.x, wc.y, wc.width, wc.height);
+    if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_MANAGED))
+    {
+        clientConfigure(c, &wc, mask, NO_CFG_FLAG);
+    }
+}
+
