@@ -1,30 +1,34 @@
-/*      $Id: spinning_cursor.c 11914 2004-08-22 22:18:50Z olivier $
- 
+/*      $Id$
+
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
         any later version.
- 
+
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
         GNU General Public License for more details.
- 
+
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- 
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
+
+
         mozilla  - (c) 1998 mozilla.org
-        xfwm4    - (c) 2002-2004 Olivier Fourdan
- 
+        xfwm4    - (c) 2002-2009 Olivier Fourdan
+
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <X11/Xlib.h>
 
+#include "spinning_cursor.h"
+
 static const unsigned char xlib_spinning_bits[] = {
     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00,
     0x0c, 0x00, 0x00, 0x00, 0x1c, 0x00, 0x00, 0x00, 0x3c, 0x00, 0x00, 0x00,
@@ -56,10 +60,10 @@ static const unsigned char xlib_spinning_mask_bits[] = {
 Cursor
 cursorCreateSpinning (Display * dpy)
 {
-    Pixmap cursor = None;
-    Pixmap mask = None;
+    Pixmap cursor;
+    Pixmap mask;
     XColor fg, bg;
-    Cursor xcursor = None;
+    Cursor xcursor;
 
     fg.pixel = 0;
     fg.red = 0;
@@ -73,11 +77,11 @@ cursorCreateSpinning (Display * dpy)
     bg.blue = 0xffff;
     bg.flags = 0xf;
 
-    cursor = XCreatePixmapFromBitmapData (dpy, DefaultRootWindow(dpy), 
+    cursor = XCreatePixmapFromBitmapData (dpy, DefaultRootWindow(dpy),
                                           (char *) xlib_spinning_bits,
                                           32, 32, 0xffffffff, 0x0, 1);
     mask   = XCreatePixmapFromBitmapData (dpy, DefaultRootWindow(dpy),
-                                          (char *) xlib_spinning_mask_bits, 
+                                          (char *) xlib_spinning_mask_bits,
                                           32, 32, 0xffffffff, 0x0, 1);
     xcursor = XCreatePixmapCursor (dpy, cursor, mask, &fg, &bg, 2, 2);
     XFreePixmap (dpy, mask);
